//https://ipinfo.io/ - cool api - possible extension

var host; //run ifconfig
var sockets = {}; // obj {socket: # packets}
var keys = []; //array of sockets - can be sorted by # packets
var total = 0; //number of packets
var statsDiv; //html div to display statistics
var time; //stores time of last packet
var bytes = 0; //number of bytes sent/recieved 
var input;

function setup() {
  createCanvas(400, 500);
  noStroke();
  textSize(16);
  input = createInput("Type IP address here"); //run ifconig
  createFileInput(gotFile); //user must select valid wireshark JSON
  statsDiv = createDiv("stats");
}

//called whenever a file is selected
function gotFile(file) {
  loadJSON(file.data, parseJSON);
}

function parseJSON(data) {
  host = input.value();
  console.log("my IP: " + host)
  //reset variables
  sockets = {}
  keys = []
  total = 0
  bytes = 0;

  //look at each packet i in data
  for (var i = 0; i < data.length; i++) {
    //create a layer variable
    var layer = data[i]._source.layers; //['_source']["layers"];

    bytes += parseInt(layer.frame["frame.len"]);
    //console.log("bytes: " + layer.frame["frame.len"])
    //if you would like to display the layers
    var layers = "";
    for (var prop in layer) {
      layers += prop + " ";
    }
    //console.log("packet " + i + ": [ " + layers + "]")
    //console.log("  length: " + layer["frame"]["frame.len"] + " bytes")
    //overall time duration
    if (i == data.length - 1) //last packet
      time = layer.frame["frame.time_relative"]

    var ip = layer.ip;
    var other, port;
    //if there is an ip layer, identify if host is source or destination IP
    //determine protocol (TCP/UDP) and get the port used by other
    //use other's IP address and port number to create a socket
    //add socket to our inventory or incremenent if already exists
    if (ip) {
      if (ip["ip.src"] == host) {
        other = ip["ip.dst"]
        port = "dstport"
      } else {
        other = ip["ip.src"]
        port = "srcport"
      }
      //console.log("  IP: " + ip["ip.src"] + " --> " + ip["ip.dst"])
      var tcp = layer["tcp"];
      var socket;

      if (tcp) {
        socket = other + ": " + tcp["tcp." + port]
          //console.log("  TCP port: " + tcp["tcp.srcport"] + " --> " + tcp["tcp.dstport"])
      } else {
        var udp = layer["udp"];
        socket = other + ": " + udp["udp." + port]
          //console.log("  UDP port: " + +udp["udp.srcport"] + " --> " + udp["udp.dstport"])
      }
      //console.log(socket)
      if (sockets[socket] === undefined) { // It's a new ip
        sockets[socket] = 1;
        keys.push(socket)
          // We've seen this word before!
      } else
        sockets[socket]++;
    } //ip
    total++;
  } //for
  //sort keys (sockets) by greatest # packets
  keys.sort(function(a, b) {
    return (sockets[b] - sockets[a]);
  });
  //display results
  for (var i = 0; i < keys.length; i++) {
    console.log(keys[i] + ': ' + sockets[keys[i]]);
  }
  bargraph();
  displayStats();
}

function bargraph() {
  background(255);
  //display top <j> IP addresses
  var j = 10;
  if (keys.length < 10)
    j = keys.length;
  for (var i = 0; i < j; i++) {
    //split IP address into subnets for color-coding
    var ipArr = split(keys[i], '.');
    if (ipArr.length > 3)
      fill(parseInt(ipArr[0]), parseInt(ipArr[1]), parseInt(ipArr[2]), 100); // parseInt(ipArr[3]));
    else
      fill(200);
    //caclulate percentage of total packets for each IP
    var perc = sockets[keys[i]] / total;
    //scale % to y-axis
    //var y = perc*height;
    var x = 3 * perc * width / 4 //- 100;
      //filter by percentage
    var spacing = 50;
    //if (perc > .01) {
    //draw rectangle along with IP address and statistics
    rect(0, i * spacing + 25, x, 20);
    fill(0);
    text(keys[i], 0, i * spacing + 20);
    fill(100);
    //console.log(sockets[keys[i]]);
    text(sockets[keys[i]], 5, i * spacing + 40)
      //console.log(nf(perc * 100, 2, 2));
    if (x < 10)
      text(nf(perc * 100, 2, 2) + "%", x + 30, i * spacing + 40);
    else
      text(nf(perc * 100, 2, 2) + "%", x + 10, i * spacing + 40);
    //}
  }
}

//displays stats as html text
function displayStats() {
  var stats = "";
  stats += "Elapsed Time: " + nf(time, -1, 2) + "s <br>";
  stats += "Packets: " + total + "<br>";
  stats += "Packets/sec: " + nf((total / time), -1, 2) + "<br>";
  stats += "Bytes: " + unit(bytes) + "<br>";
  stats += "Avg Packet Size " + unit(bytes, total, "B") + "<br>";
  stats += "Data Rate: " + unit(bytes, time, "b") + "<br>";
  statsDiv.html(stats)
}
//converts to the appropriate unit
function unit(data, t, opt) {
  var b, kb, mb, gb;
  if (t)
    data /= t;
  if (opt == 'b') {
    data *= 8;
    b = "bps"
    kb = "Kbps"
    mb = "Mbps"
    gb = "Gbps"
  } else {
    b = "B"
    kb = "KB"
    mb = "MB"
    gb = "GB"
  }
  var i = 0;
  while (data >= 1000) {
    data /= 1000;
    i++;
  }
  data = nf(data, -1, 2);
  if (i == 0) //bytes
    return data + " " + b;
  else if (i == 1) //Kb
    return data + " " + kb;
  else if (i == 2) //Mb
    return data + " " + mb;
  else
    return data + " " + gb;
}
